from rest_framework import serializers

from ...utils import EDate


class FileUploadStatusSerializer(serializers.Serializer):
    status = serializers.BooleanField()
    last_upload_date = serializers.DateTimeField()


class FileUploadStatusFilterSerializer(serializers.Serializer):
    date = serializers.DateField(required=False, default=EDate.today())



from django.contrib import admin

from .models import ImportFile, TemporaryData


@admin.register(ImportFile)
class ImportFileAdmin(admin.ModelAdmin):
    list_display = ['id', 'date', 'status', 'active', 'created_by']


@admin.register(TemporaryData)
class TemporaryDataAdmin(admin.ModelAdmin):
    list_display = ['id', 'file', 'data']



from django.utils import timezone
from django_q.models import Schedule
from drf_spectacular.utils import extend_schema
from rest_framework import viewsets

from energy_base.api.permissions import IsSuperUser
from energy_base.q_api.serializers import ScheduleSerializer


@extend_schema(tags=['admin/schedules'])
class ScheduleViewSet(viewsets.ModelViewSet):
    http_method_names = ['get', 'patch']
    queryset = Schedule.objects.all()
    serializer_class = ScheduleSerializer
    permission_classes = [IsSuperUser]


@extend_schema(tags=['admin/schedules'], request={})
class ScheduleRunNowView(viewsets.generics.UpdateAPIView):
    serializer_class = ScheduleSerializer
    http_method_names = ['patch']
    queryset = Schedule.objects.all()

    def perform_update(self, serializer):
        serializer.save(next_run=timezone.now())



from django_q.models import Schedule
from rest_framework import serializers

from energy_base.q_api.serializers import TaskSerializer


class ScheduleSerializer(serializers.ModelSerializer):
    task = TaskSerializer(read_only=True)

    class Meta:
        model = Schedule
        fields = '__all__'
        read_only_fields = ['name', 'func', 'hook', 'args', 'kwargs', 'cluster']



from rest_framework.response import Response
from rest_framework.views import APIView

from ..models import ImportFile
from ..serializers import FileUploadStatusSerializer, FileUploadStatusFilterSerializer
from ...utils.date import to_edate


class FileUploadStatusView(APIView):
    serializer_class = FileUploadStatusSerializer

    def get(self, request, *args, **kwargs):
        filter_serializer = FileUploadStatusFilterSerializer(data=request.GET)
        filter_serializer.is_valid(raise_exception=True)
        filter_date = to_edate(filter_serializer.validated_data['date'])

        status = ImportFile.objects.filter(
            date=filter_date.yesterday(),
            created_at__lte=filter_date.datetime().replace(hour=10, minute=30),
        ).exists()
        last_upload_import_file = ImportFile.objects.filter(
            created_at__date__lte=filter_date,
        ).order_by('-created_at').first()
        if last_upload_import_file:
            last_upload_datetime = str(last_upload_import_file.created_at)
        else:
            last_upload_datetime = None

        return Response({
            'status': status,
            'last_upload_datetime': last_upload_datetime,
        })



from django.db.models import Manager
from django.utils.translation import get_language, activate
from rest_framework import generics

AVAILABLE_LANGUAGES = ['uz', 'ru', 'crl']


class TranslatedListView(generics.ListAPIView):
    manager: Manager

    def get_queryset(self):
        self.update_lang()
        return self.manager.all()

    def update_lang(self):
        activate(self.get_language())

    def get_language(self):
        if lang_from_header := self.request.headers.get('accept-language'):
            if lang_from_header and lang_from_header not in AVAILABLE_LANGUAGES:
                lang_from_header = 'ru'

        return lang_from_header or get_language()
